/*
 * This file is a part of KEMET project (Library).
 * 
 * Copyright (C) 2008 MAUPIN Fabrice
 *
 * This library is free software; 
 * you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License 
 * as published by the Free Software Foundation; 
 * either version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this library; 
 * if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 */
package org.kemet.tests.integration.hieroglyph;

import org.apache.log4j.Logger;
import org.junit.Test;
import org.kemet.impl.grammar.exceptions.KemetAPIException;
import org.kemet.impl.grammar.integration.HieroglyphManager;
import org.kemet.tests.TestRoot;

/**
 * 
 * @author fabricemaupin
 * 
 * <p>
 * <b>integration tests : analyze list of hieroglyphs - output : list of type.</b>
 * </p>
 * 
 * support : uniliteral, biliteral, triliteral, phonetic complement, determiner,
 * ideogram, indicator stating of adverbial predicate proposition (ppa),
 * pronoun, preposition, noun
 * 
 * @since 0.6
 * 
 */
public class TestsToType extends TestRoot {
	static final Logger log = Logger.getLogger("org.kemet.tests");

	/**
	 * <p>
	 * <b>initialize the parameters before to execute the tests.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	public void init() {
		// nothing
	}

	/**
	 * <p>
	 * <b>uniliteral + biliteral + pc + ideograms.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test1() {
		// initialize the data tests
		sequence = "S29Y5S3Y1Y2";
		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>uniliteral + biliteral + uniliteral + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test2() {
		// initialize the data tests
		sequence = "[S29;O34]Y5[S3;N35]Y1";
		result = new String[] {
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>biliteral + biliteral + uniliteral + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test3() {
		// initialize the data tests
		sequence = "M22-M22Y5[S3;N35]Y1";
		result = new String[] {
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>biliteral + biliteral + uniliteral + determiner + biliteral.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test4() {
		// initialize the data tests
		sequence = "N35-N35-N35Y5[S3;N35]Y2Y5";
		result = new String[] {
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("BILITERAL") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			e.printStackTrace();
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>empty sequence.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test5() {
		// initialize the data tests
		sequence = "";
		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;

		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, 0);
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>biliteral + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test6() {
		// initialize the data tests
		sequence = "[W24;W24-U19;W25]Y1";
		result = new String[] {
				org.kemet.message.Messages.getString("SEVERAL_TYPES"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>biliteral + several_types + determiner for specify a ideogram.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test7() {
		// initialize the data tests
		sequence = "V4X1Z1";
		result = new String[] {
				org.kemet.message.Messages.getString("BILITERAL"),
				org.kemet.message.Messages
						.getString("FEMININE_GRAMMATICAL_KIND"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + determiner + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test8() {
		// initialize the data tests
		sequence = "A12A3Z2";
		result = new String[] {
				org.kemet.message.Messages.getString("DETERMINER"),
				org.kemet.message.Messages.getString("SEVERAL_TYPES"),
				org.kemet.message.Messages.getString("SEVERAL_TYPES") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + phonetic complement + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test9() {
		// initialize the data tests
		sequence = "D34aG1A24";
		result = new String[] {
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("DETERMINER") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + several phonetic complements + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test10() {
		// initialize the data tests
		sequence = "D34aG1V28A24";
		result = new String[] {
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("DETERMINER") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram + several phonetic complements + determiner + ideogram +
	 * several phonetic complements + determiner.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test11() {
		// initialize the data tests
		sequence = "D34aG1V28A24D34aG1V28A24";

		result = new String[] {
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("DETERMINER"),
				org.kemet.message.Messages.getString("IDEOGRAM"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("PC"),
				org.kemet.message.Messages.getString("DETERMINER") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + pronoun + preposition + uniliteral + feminine
	 * grammatical kind + determiner for specify a ideogram.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test12() {
		// initialize the data tests
		sequence = "M17G43I9G17F32X1Z1";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN"),
				org.kemet.message.Messages.getString("PREPOSITION"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages
						.getString("FEMININE_GRAMMATICAL_KIND"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + preposition + several types + pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test13() {
		// initialize the data tests
		sequence = "M17G43G17D1A1";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("PREPOSITION"),
				org.kemet.message.Messages.getString("SEVERAL_TYPES"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + pronoun + preposition + pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test14() {
		// initialize the data tests
		sequence = "M17G43A1G17V13N35";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN"),
				org.kemet.message.Messages.getString("PREPOSITION"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + demonstrative pronoun + suffix pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test15() {
		// initialize the data tests
		sequence = "M17G43M22M22O34N35";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + suffix pronoun + demonstrative pronoun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test16() {
		// initialize the data tests
		sequence = "M17G43N35M22M22";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("SUFFIX_PRONOUN"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + demonstrative pronoun + preposition + uniliteral +
	 * feminine grammatical kind + determiner for specify a ideogram .</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test17() {
		// initialize the data tests
		sequence = "M17G43G17F32X1Z1";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("DEMONSTRATIVE_PRONOUN"),
				org.kemet.message.Messages.getString("PREPOSITION"),
				org.kemet.message.Messages.getString("UNILITERAL"),
				org.kemet.message.Messages
						.getString("FEMININE_GRAMMATICAL_KIND"),
				org.kemet.message.Messages.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>indicator of PPA + noun + preposition + noun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test18() {
		// initialize the data tests
		sequence = "M17G43G29V31A1D2Z1N35O4X1M1";
		result = new String[] {
				org.kemet.message.Messages.getString("INDICATOR_PPA"),
				org.kemet.message.Messages.getString("NOUN"),
				org.kemet.message.Messages.getString("PREPOSITION"),
				org.kemet.message.Messages.getString("NOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>noun + noun.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test19() {
		// initialize the data tests
		sequence = "G29V31A1N35O4X1M1";
		result = new String[] { org.kemet.message.Messages.getString("NOUN"),
				org.kemet.message.Messages.getString("NOUN") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>preposition.</b>
	 * </p>
	 * 
	 * @since 0.6
	 */
	@Test
	public void test20() {
		// initialize the data tests
		sequence = "T28D21";
		result = new String[] { org.kemet.message.Messages
				.getString("PREPOSITION") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram (with specific determiner).</b>
	 * </p>
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test21() {
		// initialize the data tests
		sequence = "G1Z1";
		result = new String[] { org.kemet.message.Messages
				.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}

	/**
	 * <p>
	 * <b>ideogram (with specific determiner) test 2.</b>
	 * </p>
	 * 
	 * @since 0.2.3
	 */
	@Test
	public void test22() {
		// initialize the data tests
		sequence = "O1Z1";
		result = new String[] { org.kemet.message.Messages
				.getString("IDEOGRAM") };

		hieroglyph = new HieroglyphManager(sequence, gm, phInfo, TO_TYPE);

		String[] res;
		try {
			res = hieroglyph.analyze();
			assertEquals(res.length, result.length);

			if (res.length == result.length) {
				for (int i = 0; i < res.length; i++) {
					log.info("res(i) = " + res[i]);
					assertEquals(res[i], result[i]);
				}
			}
		} catch (KemetAPIException e) {
			assertTrue(false);
		}
	}
}
